!WinFadeClassType methods implementations

  MEMBER()

  INCLUDE('WinFade.Inc'),ONCE

GWL_EXSTYLE       EQUATE(-20)
WS_EX_LAYERED     EQUATE(00080000h)
ULW_ALPHA         EQUATE(00000002h)
BITSPIXEL         EQUATE(12)

Direction:FadeIn  EQUATE(0)
Direction:FadeOut EQUATE(1)

Level:Benign      EQUATE(0)
Level:User        EQUATE(1)
Level:Program     EQUATE(2)
Level:Fatal       EQUATE(3)
Level:Cancel      EQUATE(4)
Level:Notify      EQUATE(5)

  MAP
    ProcType(UNSIGNED,ULONG,BYTE,ULONG),SIGNED,PASCAL,TYPE

    CallProcByName(*ProcType,UNSIGNED,ULONG,BYTE,ULONG),SIGNED,PROC,NAME('CallProc')
    MODULE('Dummy Module')
      CallProcByAddr(LONG,UNSIGNED,ULONG,BYTE,ULONG),SIGNED,PROC,NAME('CallProc')
    END

    MODULE('WinAPI')
      LoadLibrary(LONG),LONG,PASCAL,PROC,NAME('LoadLibraryA')
      FreeLibrary(LONG),SIGNED,PASCAL,PROC
      GetProcAddress(LONG,LONG),LONG,PASCAL

      SetWindowLong(UNSIGNED hWnd,SIGNED Style,LONG Flags),LONG,PASCAL,PROC,NAME('SetWindowLongA')
      GetWindowLong(UNSIGNED hWnd,SIGNED Style),LONG,PASCAL,PROC,NAME('GetWindowLongA')

      GetDC(UNSIGNED),UNSIGNED,PASCAL
      GetDeviceCaps(UNSIGNED,SIGNED),SIGNED,PASCAL
    END
  END

!============================ Public Methods ==================================

WinFadeClassType.Init PROCEDURE(LONG OpenCloseTime = 50,LONG FadeStep = 4)

LOC:cstDLLName   CSTRING(100),AUTO
LOC:cstProcName  CSTRING(100),AUTO

  CODE
  IF SELF.Initialized = True THEN RETURN(1).

  SELF.Window &= SYSTEM{PROP:Target}
  SELF.WindowHandle = SELF.Window{PROP:Handle}

  IF GetDeviceCaps(GetDC(SELF.WindowHandle),BITSPIXEL) < 16 THEN RETURN(1).

  SELF.OpenCloseTime = OpenCloseTime
  SELF.FadeStep      = FadeStep
  IF SELF.FadeStep < 1 OR SELF.FadeStep > 100
    SELF.FadeStep = 1
  END

  LOC:cstDLLName = 'user32.dll'
  SELF.DLLInstance = LoadLibrary(ADDRESS(LOC:cstDLLName))
  IF SELF.DLLInstance = 0 THEN RETURN(1).

  LOC:cstProcName = 'SetLayeredWindowAttributes'
  SELF.ProcAddress = GetProcAddress(SELF.DLLInstance,ADDRESS(LOC:cstProcName))
  IF SELF.ProcAddress = 0 THEN RETURN(1).

  SELF.FadeProgress     = False
  SELF.TransparentLevel = 0
  SELF.FadeDirection    = Direction:FadeIn

  SELF.Initialized = True

  SELF.SetTransparant(SELF.TransparentLevel)

  REGISTEREVENT(EVENT:OpenWindow,ADDRESS(SELF.TakeEvents),ADDRESS(SELF))
  REGISTEREVENT(EVENT:Timer,ADDRESS(SELF.TakeEvents),ADDRESS(SELF))
  REGISTEREVENT(EVENT:CloseWindow,ADDRESS(SELF.TakeEvents),ADDRESS(SELF))

  RETURN(0)
!------------------------------------------------------------------------------

WinFadeClassType.SetTransparant PROCEDURE(LONG Level) 
  CODE
  IF SELF.Initialized = False THEN RETURN.

  SELF.TransparentLevel = Level
  SetWindowLong(SELF.WindowHandle,GWL_EXSTYLE,BOR(GetWindowLong(SELF.WindowHandle,GWL_EXSTYLE),WS_EX_LAYERED))
  CallProcByAddr(SELF.ProcAddress,SELF.WindowHandle,0,(255 * Level) / 100,ULW_ALPHA)

  RETURN
!------------------------------------------------------------------------------

!=========================== Protected Methods ================================

WinFadeClassType.TakeEvents PROCEDURE
  CODE
  IF SELF.Initialized = False THEN RETURN(Level:Benign).

  CASE EVENT()
  OF EVENT:OpenWindow
    SELF.FadeProgress = True
    SELF.OldTimer = SELF.Window{PROP:Timer}
    SELF.Window{PROP:Timer} = SELF.OpenCloseTime * SELF.FadeStep / 100
    IF SELF.Window{PROP:Timer} = 0
      SELF.Window{PROP:Timer} = 1
    END
  OF EVENT:Timer
    IF SELF.FadeProgress = True
      CASE SELF.FadeDirection
      OF Direction:FadeIn
        SELF.TransparentLevel += SELF.FadeStep
      OF Direction:FadeOut
        SELF.TransparentLevel -= SELF.FadeStep
      END
      SELF.SetTransparant(SELF.TransparentLevel)
      CASE SELF.FadeDirection
      OF Direction:FadeIn
        IF SELF.TransparentLevel >= 100
          SELF.Window{PROP:Timer} = SELF.OldTimer
          SELF.FadeProgress = False
        END
        RETURN(Level:Notify)  ! CYCLE
      OF Direction:FadeOut
        IF SELF.TransparentLevel <= 0
          SELF.Window{PROP:Timer} = SELF.OldTimer
          SELF.FadeProgress = False
          SELF.Kill()
          POST(EVENT:CloseWindow)
        END
        RETURN(Level:Notify)  ! CYCLE
      END
    END
  OF EVENT:CloseWindow
    SELF.FadeProgress = True
    SELF.OldTimer = SELF.Window{PROP:Timer}
    SELF.Window{PROP:Timer} = SELF.OpenCloseTime * SELF.FadeStep / 100
    IF SELF.Window{PROP:Timer} = 0
      SELF.Window{PROP:Timer} = 1
    END
    SELF.FadeDirection = Direction:FadeOut
    RETURN(Level:Notify)  ! CYCLE
  END
  RETURN(Level:Benign)

!------------------------------------------------------------------------------

WinFadeClassType.Kill PROCEDURE
  CODE
  IF SELF.Initialized = False THEN RETURN.

  FreeLibrary(SELF.DLLInstance)

  UNREGISTEREVENT(EVENT:CloseWindow,ADDRESS(SELF.TakeEvents),ADDRESS(SELF))
  UNREGISTEREVENT(EVENT:Timer,ADDRESS(SELF.TakeEvents),ADDRESS(SELF))
  UNREGISTEREVENT(EVENT:OpenWindow,ADDRESS(SELF.TakeEvents),ADDRESS(SELF))

  SELF.Initialized = False

  RETURN
!------------------------------------------------------------------------------


CallProcByName PROCEDURE(*ProcType Proc,UNSIGNED hWnd,ULONG crKey,BYTE bAlpha,ULONG Flags)
  CODE
! Launch procedure by prototype:
! SetLayeredWindowAttributes(UNSIGNED hWnd,ULONG crKey,BYTE bAlpha,ULONG Flags),SIGNED,RAW,PASCAL,PROC
  RETURN(Proc(hWnd,crKey,bAlpha,Flags))
!------------------------------------------------------------------------------
